@@ -15,8 +15,11 @@
 
 import os
 import re
-import sys
-import warnings
+
+from kmip.core import enums
+from kmip.pie import client
+from kmip.pie import objects
+from kmip.pie.client import ProxyKmipClient as KmipClient
 
 # Dynamically set __version__
 version_path = os.path.join(os.path.dirname(
@@ -26,13 +29,12 @@ with open(version_path, 'r') as version_file:
     __version__ = mo.group(1)
 
 
-__all__ = ['core', 'demos', 'services']
-
-
-if sys.version_info[:2] == (2, 6):
-    warnings.simplefilter("always")
-    warnings.warn(
-        ("Please use a newer version of Python (2.7.9+ preferred). PyKMIP "
-         "support for Python 2.6 will be deprecated in the future."),
-        PendingDeprecationWarning)
-    warnings.simplefilter("default")
+__all__ = [
+    'client',
+    'core',
+    'demos',
+    'enums',
+    'KmipClient',
+    'objects',
+    'services'
+]
