@@ -678,7 +678,7 @@ class Boolean(Base):
         """
         try:
             value = unpack('!Q', istream.read(self.LENGTH))[0]
-        except:
+        except Exception:
             self.logger.error("Error reading boolean value from buffer")
             raise
 
@@ -713,7 +713,7 @@ class Boolean(Base):
         """
         try:
             ostream.write(pack('!Q', self.value))
-        except:
+        except Exception:
             self.logger.error("Error writing boolean value to buffer")
             raise
 
@@ -810,11 +810,7 @@ class TextString(Base):
     def write_value(self, ostream):
         # Write string to stream
         for char in self.value:
-            if sys.version < '3':
-                c = char
-            else:
-                c = char.encode()
-            ostream.write(pack(self.BYTE_FORMAT, c))
+            ostream.write(pack(self.BYTE_FORMAT, char.encode()))
 
         # Write padding to stream
         for _ in range(self.padding_length):
@@ -829,17 +825,16 @@ class TextString(Base):
 
     def __validate(self):
         if self.value is not None:
-            data_type = type(self.value)
-            if data_type is not str:
+            if not isinstance(self.value, six.string_types):
                 msg = ErrorStrings.BAD_EXP_RECV
                 raise TypeError(msg.format('TextString', 'value', str,
-                                           data_type))
+                                           type(self.value)))
 
     def __repr__(self):
         return "{0}(value={1})".format(type(self).__name__, repr(self.value))
 
     def __str__(self):
-        return "{0}".format(repr(self.value))
+        return "{0}".format(str(self.value))
 
     def __eq__(self, other):
         if isinstance(other, TextString):
