@@ -14,7 +14,6 @@
 # under the License.
 
 import logging
-import os
 import sys
 
 from kmip.core import enums
@@ -23,6 +22,8 @@ from kmip.pie import client
 
 
 if __name__ == '__main__':
+    logger = utils.build_console_logger(logging.INFO)
+
     # Build and parse arguments
     parser = utils.build_cli_parser(enums.Operation.DESTROY)
     opts, args = parser.parse_args(sys.argv[1:])
@@ -32,15 +33,9 @@ if __name__ == '__main__':
 
     # Exit early if the UUID is not specified
     if uid is None:
-        logging.debug('No UUID provided, exiting early from demo')
+        logger.error('No UUID provided, exiting early from demo')
         sys.exit()
 
-    # Build and setup logging and needed factories
-    f_log = os.path.join(os.path.dirname(__file__), os.pardir, os.pardir,
-                         'logconfig.ini')
-    logging.config.fileConfig(f_log)
-    logger = logging.getLogger(__name__)
-
     # Build the client and connect to the server
     with client.ProxyKmipClient(config=config) as client:
         try:
