@@ -22,7 +22,7 @@ from kmip.core.attributes import CryptographicAlgorithm
 from kmip.core.attributes import CryptographicLength
 
 from kmip.core.enums import AttributeType
-from kmip.core.enums import CertificateTypeEnum
+from kmip.core.enums import CertificateType
 from kmip.core.enums import CryptographicAlgorithm as CryptoAlgorithmEnum
 from kmip.core.enums import CryptographicUsageMask
 from kmip.core.enums import ObjectType
@@ -214,15 +214,62 @@ def build_cli_parser(operation=None):
                   "SECRET_DATA"))
     elif operation is Operation.DISCOVER_VERSIONS:
         parser.add_option(
-                "-v",
-                "--protocol-versions",
-                action="store",
-                type="str",
-                default=None,
-                dest="protocol_versions",
-                help=("Protocol versions supported by client. "
-                      "ex. '1.1,1.2 1.3'"))
-
+            "-v",
+            "--protocol-versions",
+            action="store",
+            type="str",
+            default=None,
+            dest="protocol_versions",
+            help=("Protocol versions supported by client. "
+                  "ex. '1.1,1.2 1.3'"))
+    elif operation is Operation.MAC:
+        parser.add_option(
+            "-i",
+            "--uuid",
+            action="store",
+            type="str",
+            default=None,
+            dest="uuid",
+            help="The unique ID of the managed object that is the key"
+                 "to use for the MAC operation")
+        parser.add_option(
+            "-a",
+            "--algorithm",
+            action="store",
+            type="str",
+            default=None,
+            dest="algorithm",
+            help="Encryption algorithm for the secret (e.g., AES)")
+    elif operation is Operation.ENCRYPT:
+        parser.add_option(
+            "-m",
+            "--message",
+            action="store",
+            type="str",
+            default="This is a secret message.",
+            dest="message",
+            help="The message to encrypt."
+        )
+    elif operation is Operation.DECRYPT:
+        parser.add_option(
+            "-i",
+            "--uuid",
+            action="store",
+            type="str",
+            default=None,
+            dest="uuid",
+            help="The unique ID of the decryption key to use to decrypt the "
+                 "cipher text."
+        )
+        parser.add_option(
+            "-m",
+            "--message",
+            action="store",
+            type="str",
+            default=None,
+            dest="message",
+            help="The cipher text to decrypt."
+        )
     return parser
 
 
@@ -255,7 +302,7 @@ def build_object(logger, object_type, key_format_type):
     if object_type == ObjectType.CERTIFICATE:
         value = build_secret_value(logger, object_type)
         return Certificate(
-            certificate_type=CertificateTypeEnum.X_509,
+            certificate_type=CertificateType.X_509,
             certificate_value=value)
     else:
         return build_key(logger, object_type, key_format_type)
