@@ -19,6 +19,7 @@ from kmip.services.results import CreateKeyPairResult
 from kmip.services.results import DestroyResult
 from kmip.services.results import DiscoverVersionsResult
 from kmip.services.results import GetResult
+from kmip.services.results import GetAttributesResult
 from kmip.services.results import GetAttributeListResult
 from kmip.services.results import LocateResult
 from kmip.services.results import OperationResult
@@ -26,6 +27,7 @@ from kmip.services.results import QueryResult
 from kmip.services.results import RegisterResult
 from kmip.services.results import RekeyKeyPairResult
 from kmip.services.results import RevokeResult
+from kmip.services.results import MACResult
 
 from kmip.core import attributes as attr
 
@@ -46,18 +48,7 @@ from kmip.core.messages.contents import ProtocolVersion
 
 from kmip.core.messages import messages
 
-from kmip.core.messages.payloads import activate
-from kmip.core.messages.payloads import create
-from kmip.core.messages.payloads import create_key_pair
-from kmip.core.messages.payloads import destroy
-from kmip.core.messages.payloads import discover_versions
-from kmip.core.messages.payloads import get
-from kmip.core.messages.payloads import get_attribute_list
-from kmip.core.messages.payloads import locate
-from kmip.core.messages.payloads import query
-from kmip.core.messages.payloads import rekey_key_pair
-from kmip.core.messages.payloads import register
-from kmip.core.messages.payloads import revoke
+from kmip.core.messages import payloads
 
 from kmip.services.server.kmip_protocol import KMIPProtocol
 
@@ -210,6 +201,8 @@ class KMIPProxy(KMIP):
         self.logger.debug("KMIPProxy suppress_ragged_eofs: {0}".format(
             self.suppress_ragged_eofs))
 
+        last_error = None
+
         for host in self.host_list:
             self.host = host
             sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
@@ -221,11 +214,13 @@ class KMIPProxy(KMIP):
                 self.logger.error("An error occurred while connecting to "
                                   "appliance " + self.host)
                 self.socket.close()
+                last_error = e
             else:
                 return
 
         self.socket = None
-        raise e
+        if last_error:
+            raise last_error
 
     def _create_socket(self, sock):
         self.socket = ssl.wrap_socket(
@@ -246,8 +241,13 @@ class KMIPProxy(KMIP):
     def close(self):
         # Shutdown and close the socket.
         if self.socket:
-            self.socket.shutdown(socket.SHUT_RDWR)
-            self.socket.close()
+            try:
+                self.socket.shutdown(socket.SHUT_RDWR)
+                self.socket.close()
+            except OSError:
+                # Can be thrown if the socket is not actually connected to
+                # anything. In this case, ignore the error.
+                pass
             self.socket = None
 
     def create(self, object_type, template_attribute, credential=None):
@@ -284,6 +284,88 @@ class KMIPProxy(KMIP):
         """
         return self._activate(uuid, credential=credential)
 
+    def derive_key(self,
+                   object_type,
+                   unique_identifiers,
+                   derivation_method,
+                   derivation_parameters,
+                   template_attribute,
+                   credential=None):
+        """
+        Derive a new key or secret data from an existing managed object.
+
+        Args:
+            object_type (ObjectType): An ObjectType enumeration specifying
+                what type of object to create. Required.
+            unique_identifiers (list): A list of strings specifying the unique
+                IDs of the existing managed objects to use for key derivation.
+                Required.
+            derivation_method (DerivationMethod): A DerivationMethod
+                enumeration specifying what key derivation method to use.
+                Required.
+            derivation_parameters (DerivationParameters): A
+                DerivationParameters struct containing the settings and
+                options to use for key derivation.
+            template_attribute (TemplateAttribute): A TemplateAttribute struct
+                containing the attributes to set on the newly derived object.
+            credential (Credential): A Credential struct containing a set of
+                authorization parameters for the operation. Optional, defaults
+                to None.
+
+        Returns:
+            dict: The results of the derivation operation, containing the
+                following key/value pairs:
+
+                Key                  | Value
+                ---------------------|-----------------------------------------
+                'unique_identifier'  | (string) The unique ID of the newly
+                                     | derived object.
+                'template_attribute' | (TemplateAttribute) A struct containing
+                                     | any attributes set on the newly derived
+                                     | object.
+                'result_status'      | (ResultStatus) An enumeration indicating
+                                     | the status of the operation result.
+                'result_reason'      | (ResultReason) An enumeration providing
+                                     | context for the result status.
+                'result_message'     | (string) A message providing additional
+                                     | context for the operation result.
+        """
+        operation = Operation(OperationEnum.DERIVE_KEY)
+        request_payload = payloads.DeriveKeyRequestPayload(
+            object_type=object_type,
+            unique_identifiers=unique_identifiers,
+            derivation_method=derivation_method,
+            derivation_parameters=derivation_parameters,
+            template_attribute=template_attribute
+        )
+        batch_item = messages.RequestBatchItem(
+            operation=operation,
+            request_payload=request_payload
+        )
+
+        request = self._build_request_message(credential, [batch_item])
+        response = self._send_and_receive_message(request)
+        batch_item = response.batch_items[0]
+        payload = batch_item.response_payload
+
+        result = {}
+
+        if payload:
+            result['unique_identifier'] = payload.unique_identifier
+            result['template_attribute'] = payload.template_attribute
+
+        result['result_status'] = batch_item.result_status.value
+        try:
+            result['result_reason'] = batch_item.result_reason.value
+        except Exception:
+            result['result_reason'] = batch_item.result_reason
+        try:
+            result['result_message'] = batch_item.result_message.value
+        except Exception:
+            result['result_message'] = batch_item.result_message
+
+        return result
+
     def get(self, uuid=None, key_format_type=None, key_compression_type=None,
             key_wrapping_specification=None, credential=None):
         return self._get(
@@ -293,6 +375,32 @@ class KMIPProxy(KMIP):
             key_wrapping_specification=key_wrapping_specification,
             credential=credential)
 
+    def get_attributes(self, uuid=None, attribute_names=None):
+        """
+        Send a GetAttributes request to the server.
+
+        Args:
+            uuid (string): The ID of the managed object with which the
+                retrieved attributes should be associated. Optional, defaults
+                to None.
+            attribute_names (list): A list of AttributeName values indicating
+                what object attributes the client wants from the server.
+                Optional, defaults to None.
+
+        Returns:
+            result (GetAttributesResult): A structure containing the results
+                of the operation.
+        """
+        batch_item = self._build_get_attributes_batch_item(
+            uuid,
+            attribute_names
+        )
+
+        request = self._build_request_message(None, [batch_item])
+        response = self._send_and_receive_message(request)
+        results = self._process_batch_items(response)
+        return results[0]
+
     def get_attribute_list(self, uid=None):
         """
         Send a GetAttributeList request to the server.
@@ -312,13 +420,16 @@ class KMIPProxy(KMIP):
         results = self._process_batch_items(response)
         return results[0]
 
-    def revoke(self, uuid, reason, message=None, credential=None):
-        return self._revoke(unique_identifier=uuid,
-                            revocation_code=reason,
-                            revocation_message=message,
-                            credential=credential)
+    def revoke(self, revocation_reason, uuid=None, revocation_message=None,
+               compromise_occurrence_date=None, credential=None):
+        return self._revoke(
+            unique_identifier=uuid,
+            revocation_reason=revocation_reason,
+            revocation_message=revocation_message,
+            compromise_occurrence_date=compromise_occurrence_date,
+            credential=credential)
 
-    def destroy(self, uuid, credential=None):
+    def destroy(self, uuid=None, credential=None):
         return self._destroy(unique_identifier=uuid,
                              credential=credential)
 
@@ -391,6 +502,308 @@ class KMIPProxy(KMIP):
             results = self._process_batch_items(response)
             return results[0]
 
+    def encrypt(self,
+                data,
+                unique_identifier=None,
+                cryptographic_parameters=None,
+                iv_counter_nonce=None,
+                credential=None):
+        """
+        Encrypt data using the specified encryption key and parameters.
+
+        Args:
+            data (bytes): The bytes to encrypt. Required.
+            unique_identifier (string): The unique ID of the encryption key
+                to use. Optional, defaults to None.
+            cryptographic_parameters (CryptographicParameters): A structure
+                containing various cryptographic settings to be used for the
+                encryption. Optional, defaults to None.
+            iv_counter_nonce (bytes): The bytes to use for the IV/counter/
+                nonce, if needed by the encryption algorithm and/or cipher
+                mode. Optional, defaults to None.
+            credential (Credential): A credential object containing a set of
+                authorization parameters for the operation. Optional, defaults
+                to None.
+
+        Returns:
+            dict: The results of the encrypt operation, containing the
+                following key/value pairs:
+
+                Key                 | Value
+                --------------------|-----------------------------------------
+                'unique_identifier' | (string) The unique ID of the encryption
+                                    | key used to encrypt the data.
+                'data'              | (bytes) The encrypted data.
+                'iv_counter_nonce'  | (bytes) The IV/counter/nonce used for
+                                    | the encryption, if autogenerated.
+                'result_status'     | (ResultStatus) An enumeration indicating
+                                    | the status of the operation result.
+                'result_reason'     | (ResultReason) An enumeration providing
+                                    | context for the result status.
+                'result_message'    | (string) A message providing additional
+                                    | context for the operation result.
+        """
+        operation = Operation(OperationEnum.ENCRYPT)
+
+        request_payload = payloads.EncryptRequestPayload(
+            unique_identifier=unique_identifier,
+            data=data,
+            cryptographic_parameters=cryptographic_parameters,
+            iv_counter_nonce=iv_counter_nonce
+        )
+        batch_item = messages.RequestBatchItem(
+            operation=operation,
+            request_payload=request_payload
+        )
+
+        request = self._build_request_message(credential, [batch_item])
+        response = self._send_and_receive_message(request)
+        batch_item = response.batch_items[0]
+        payload = batch_item.response_payload
+
+        result = {}
+
+        if payload:
+            result['unique_identifier'] = payload.unique_identifier
+            result['data'] = payload.data
+            result['iv_counter_nonce'] = payload.iv_counter_nonce
+
+        result['result_status'] = batch_item.result_status.value
+        try:
+            result['result_reason'] = batch_item.result_reason.value
+        except Exception:
+            result['result_reason'] = batch_item.result_reason
+        try:
+            result['result_message'] = batch_item.result_message.value
+        except Exception:
+            result['result_message'] = batch_item.result_message
+
+        return result
+
+    def decrypt(self,
+                data,
+                unique_identifier=None,
+                cryptographic_parameters=None,
+                iv_counter_nonce=None,
+                credential=None):
+        """
+        Decrypt data using the specified decryption key and parameters.
+
+        Args:
+            data (bytes): The bytes to decrypt. Required.
+            unique_identifier (string): The unique ID of the decryption key
+                to use. Optional, defaults to None.
+            cryptographic_parameters (CryptographicParameters): A structure
+                containing various cryptographic settings to be used for the
+                decryption. Optional, defaults to None.
+            iv_counter_nonce (bytes): The bytes to use for the IV/counter/
+                nonce, if needed by the decryption algorithm and/or cipher
+                mode. Optional, defaults to None.
+            credential (Credential): A credential object containing a set of
+                authorization parameters for the operation. Optional, defaults
+                to None.
+
+        Returns:
+            dict: The results of the decrypt operation, containing the
+                following key/value pairs:
+
+                Key                 | Value
+                --------------------|-----------------------------------------
+                'unique_identifier' | (string) The unique ID of the decryption
+                                    | key used to decrypt the data.
+                'data'              | (bytes) The decrypted data.
+                'result_status'     | (ResultStatus) An enumeration indicating
+                                    | the status of the operation result.
+                'result_reason'     | (ResultReason) An enumeration providing
+                                    | context for the result status.
+                'result_message'    | (string) A message providing additional
+                                    | context for the operation result.
+        """
+        operation = Operation(OperationEnum.DECRYPT)
+
+        request_payload = payloads.DecryptRequestPayload(
+            unique_identifier=unique_identifier,
+            data=data,
+            cryptographic_parameters=cryptographic_parameters,
+            iv_counter_nonce=iv_counter_nonce
+        )
+        batch_item = messages.RequestBatchItem(
+            operation=operation,
+            request_payload=request_payload
+        )
+
+        request = self._build_request_message(credential, [batch_item])
+        response = self._send_and_receive_message(request)
+        batch_item = response.batch_items[0]
+        payload = batch_item.response_payload
+
+        result = {}
+
+        if payload:
+            result['unique_identifier'] = payload.unique_identifier
+            result['data'] = payload.data
+
+        result['result_status'] = batch_item.result_status.value
+        try:
+            result['result_reason'] = batch_item.result_reason.value
+        except Exception:
+            result['result_reason'] = batch_item.result_reason
+        try:
+            result['result_message'] = batch_item.result_message.value
+        except Exception:
+            result['result_message'] = batch_item.result_message
+
+        return result
+
+    def signature_verify(self,
+                         message,
+                         signature,
+                         unique_identifier=None,
+                         cryptographic_parameters=None,
+                         credential=None):
+        """
+        Verify a message signature using the specified signing key.
+
+        Args:
+            message (bytes): The bytes of the signed message. Required.
+            signature (bytes): The bytes of the message signature. Required.
+            unique_identifier (string): The unique ID of the signing key to
+                use. Optional, defaults to None.
+            cryptographic_parameters (CryptographicParameters): A structure
+                containing various cryptographic settings to be used for
+                signature verification. Optional, defaults to None.
+            credential (Credential): A credential object containing a set of
+                authorization parameters for the operation. Optional, defaults
+                to None.
+
+        Returns:
+            dict: The results of the signature verify operation, containing the
+                following key/value pairs:
+
+                Key                  | Value
+                ---------------------|-----------------------------------------
+                'unique_identifier'  | (string) The unique ID of the signing
+                                     | key used to verify the signature.
+                'validity_indicator' | (ValidityIndicator) An enumeration
+                                     | indicating the result of signature
+                                     | verification.
+                'result_status'      | (ResultStatus) An enumeration indicating
+                                     | the status of the operation result.
+                'result_reason'      | (ResultReason) An enumeration providing
+                                     | context for the result status.
+                'result_message'     | (string) A message providing additional
+                                     | context for the operation result.
+        """
+        operation = Operation(OperationEnum.SIGNATURE_VERIFY)
+
+        request_payload = payloads.SignatureVerifyRequestPayload(
+            unique_identifier=unique_identifier,
+            cryptographic_parameters=cryptographic_parameters,
+            data=message,
+            signature_data=signature
+        )
+        batch_item = messages.RequestBatchItem(
+            operation=operation,
+            request_payload=request_payload
+        )
+
+        request = self._build_request_message(credential, [batch_item])
+        response = self._send_and_receive_message(request)
+        batch_item = response.batch_items[0]
+        payload = batch_item.response_payload
+
+        result = {}
+
+        if payload:
+            result['unique_identifier'] = payload.unique_identifier
+            result['validity_indicator'] = payload.validity_indicator
+
+        result['result_status'] = batch_item.result_status.value
+        try:
+            result['result_reason'] = batch_item.result_reason.value
+        except Exception:
+            result['result_reason'] = batch_item.result_reason
+        try:
+            result['result_message'] = batch_item.result_message.value
+        except Exception:
+            result['result_message'] = batch_item.result_message
+
+        return result
+
+    def sign(self, data, unique_identifier=None,
+             cryptographic_parameters=None, credential=None):
+        """
+        Sign specified data using a specified signing key.
+
+        Args:
+            data (bytes): Data to be signed. Required.
+            unique_identifier (string): The unique ID of the signing
+                key to be used. Optional, defaults to None.
+            cryptographic_parameters (CryptographicParameters): A structure
+                containing various cryptographic settings to be used for
+                creating the signature. Optional, defaults to None.
+            credential (Credential): A credential object containing a set of
+                authorization parameters for the operation. Optional, defaults
+                to None.
+        Returns:
+            dict: The results of the sign operation, containing the
+                following key/value pairs:
+
+            Key                  | Value
+            ---------------------|-----------------------------------------
+            'unique_identifier'  | (string) The unique ID of the signing
+                                 | key used to create the signature
+            'signature'          | (bytes) The bytes of the signature
+            'result_status'      | (ResultStatus) An enumeration indicating
+                                 | the status of the operation result
+            'result_reason'      | (ResultReason) An enumeration providing
+                                 | context for the result status.
+            'result_message'     | (string) A message providing additional
+                                 | context for the operation result.
+        """
+        operation = Operation(OperationEnum.SIGN)
+
+        request_payload = payloads.SignRequestPayload(
+            unique_identifier=unique_identifier,
+            cryptographic_parameters=cryptographic_parameters,
+            data=data
+        )
+
+        batch_item = messages.RequestBatchItem(
+            operation=operation,
+            request_payload=request_payload
+        )
+
+        request = self._build_request_message(credential, [batch_item])
+        response = self._send_and_receive_message(request)
+        batch_item = response.batch_items[0]
+        payload = batch_item.response_payload
+
+        result = {}
+
+        if payload:
+            result['unique_identifier'] = payload.unique_identifier
+            result['signature'] = payload.signature_data
+        result['result_status'] = batch_item.result_status.value
+        try:
+            result['result_reason'] = batch_item.result_reason.value
+        except Exception:
+            result['result_reason'] = batch_item.result_reason
+        try:
+            result['result_message'] = batch_item.result_message.value
+        except Exception:
+            result['result_message'] = batch_item.result_message
+
+        return result
+
+    def mac(self, data, unique_identifier=None,
+            cryptographic_parameters=None, credential=None):
+        return self._mac(
+            data=data,
+            unique_identifier=unique_identifier,
+            cryptographic_parameters=cryptographic_parameters,
+            credential=credential)
+
     def _create(self,
                 object_type=None,
                 template_attribute=None,
@@ -400,7 +813,7 @@ class KMIPProxy(KMIP):
         if object_type is None:
             raise ValueError('object_type cannot be None')
 
-        req_pl = create.CreateRequestPayload(
+        req_pl = payloads.CreateRequestPayload(
             object_type=object_type,
             template_attribute=template_attribute)
         batch_item = messages.RequestBatchItem(operation=operation,
@@ -436,7 +849,7 @@ class KMIPProxy(KMIP):
                                           private_key_template_attribute=None,
                                           public_key_template_attribute=None):
         operation = Operation(OperationEnum.CREATE_KEY_PAIR)
-        payload = create_key_pair.CreateKeyPairRequestPayload(
+        payload = payloads.CreateKeyPairRequestPayload(
             common_template_attribute=common_template_attribute,
             private_key_template_attribute=private_key_template_attribute,
             public_key_template_attribute=public_key_template_attribute)
@@ -450,7 +863,7 @@ class KMIPProxy(KMIP):
                                          private_key_template_attribute=None,
                                          public_key_template_attribute=None):
         operation = Operation(OperationEnum.REKEY_KEY_PAIR)
-        payload = rekey_key_pair.RekeyKeyPairRequestPayload(
+        payload = payloads.RekeyKeyPairRequestPayload(
             private_key_uuid, offset,
             common_template_attribute=common_template_attribute,
             private_key_template_attribute=private_key_template_attribute,
@@ -461,14 +874,30 @@ class KMIPProxy(KMIP):
 
     def _build_query_batch_item(self, query_functions=None):
         operation = Operation(OperationEnum.QUERY)
-        payload = query.QueryRequestPayload(query_functions)
+        payload = payloads.QueryRequestPayload(query_functions)
         batch_item = messages.RequestBatchItem(
             operation=operation, request_payload=payload)
         return batch_item
 
+    def _build_get_attributes_batch_item(
+            self,
+            uuid=None,
+            attribute_names=None
+    ):
+        operation = Operation(OperationEnum.GET_ATTRIBUTES)
+        payload = payloads.GetAttributesRequestPayload(
+            uuid,
+            attribute_names
+        )
+        batch_item = messages.RequestBatchItem(
+            operation=operation,
+            request_payload=payload
+        )
+        return batch_item
+
     def _build_get_attribute_list_batch_item(self, uid=None):
         operation = Operation(OperationEnum.GET_ATTRIBUTE_LIST)
-        payload = get_attribute_list.GetAttributeListRequestPayload(uid)
+        payload = payloads.GetAttributeListRequestPayload(uid)
         batch_item = messages.RequestBatchItem(
             operation=operation, request_payload=payload)
         return batch_item
@@ -476,7 +905,7 @@ class KMIPProxy(KMIP):
     def _build_discover_versions_batch_item(self, protocol_versions=None):
         operation = Operation(OperationEnum.DISCOVER_VERSIONS)
 
-        payload = discover_versions.DiscoverVersionsRequestPayload(
+        payload = payloads.DiscoverVersionsRequestPayload(
             protocol_versions)
 
         batch_item = messages.RequestBatchItem(
@@ -499,6 +928,8 @@ class KMIPProxy(KMIP):
             return self._process_response_error
         elif operation == OperationEnum.CREATE_KEY_PAIR:
             return self._process_create_key_pair_batch_item
+        elif operation == OperationEnum.GET_ATTRIBUTES:
+            return self._process_get_attributes_batch_item
         elif operation == OperationEnum.GET_ATTRIBUTE_LIST:
             return self._process_get_attribute_list_batch_item
         elif operation == OperationEnum.REKEY_KEY_PAIR:
@@ -511,6 +942,24 @@ class KMIPProxy(KMIP):
             raise ValueError("no processor for operation: {0}".format(
                 operation))
 
+    def _process_get_attributes_batch_item(self, batch_item):
+        payload = batch_item.response_payload
+
+        uuid = None
+        attributes = None
+
+        if payload:
+            uuid = payload.unique_identifier
+            attributes = payload.attributes
+
+        return GetAttributesResult(
+            batch_item.result_status,
+            batch_item.result_reason,
+            batch_item.result_message,
+            uuid,
+            attributes
+        )
+
     def _process_get_attribute_list_batch_item(self, batch_item):
         payload = batch_item.response_payload
 
@@ -518,7 +967,7 @@ class KMIPProxy(KMIP):
         names = None
 
         if payload:
-            uid = payload.uid
+            uid = payload.unique_identifier
             names = payload.attribute_names
 
         return GetAttributeListResult(
@@ -610,25 +1059,15 @@ class KMIPProxy(KMIP):
              credential=None):
         operation = Operation(OperationEnum.GET)
 
-        uuid = None
-        kft = None
-        kct = None
-        kws = None
-
-        if unique_identifier is not None:
-            uuid = attr.UniqueIdentifier(unique_identifier)
         if key_format_type is not None:
-            kft = get.GetRequestPayload.KeyFormatType(key_format_type.value)
-        if key_compression_type is not None:
-            kct = key_compression_type
-            kct = get.GetRequestPayload.KeyCompressionType(kct)
-        if key_wrapping_specification is not None:
-            kws = objects.KeyWrappingSpecification(key_wrapping_specification)
+            key_format_type = key_format_type.value
 
-        req_pl = get.GetRequestPayload(unique_identifier=uuid,
-                                       key_format_type=kft,
-                                       key_compression_type=kct,
-                                       key_wrapping_specification=kws)
+        req_pl = payloads.GetRequestPayload(
+            unique_identifier=unique_identifier,
+            key_format_type=key_format_type,
+            key_compression_type=key_compression_type,
+            key_wrapping_specification=key_wrapping_specification
+        )
 
         batch_item = messages.RequestBatchItem(operation=operation,
                                                request_payload=req_pl)
@@ -665,7 +1104,7 @@ class KMIPProxy(KMIP):
         if unique_identifier is not None:
             uuid = attr.UniqueIdentifier(unique_identifier)
 
-        payload = activate.ActivateRequestPayload(unique_identifier=uuid)
+        payload = payloads.ActivateRequestPayload(unique_identifier=uuid)
 
         batch_item = messages.RequestBatchItem(operation=operation,
                                                request_payload=payload)
@@ -698,7 +1137,7 @@ class KMIPProxy(KMIP):
         if unique_identifier is not None:
             uuid = attr.UniqueIdentifier(unique_identifier)
 
-        payload = destroy.DestroyRequestPayload(unique_identifier=uuid)
+        payload = payloads.DestroyRequestPayload(unique_identifier=uuid)
 
         batch_item = messages.RequestBatchItem(operation=operation,
                                                request_payload=payload)
@@ -722,20 +1161,21 @@ class KMIPProxy(KMIP):
                                payload_unique_identifier)
         return result
 
-    def _revoke(self, unique_identifier=None, revocation_code=None,
-                revocation_message=None, credential=None):
+    def _revoke(self, unique_identifier=None, revocation_reason=None,
+                revocation_message=None, compromise_occurrence_date=None,
+                credential=None):
         operation = Operation(OperationEnum.REVOKE)
 
-        reason = objects.RevocationReason(code=revocation_code,
+        reason = objects.RevocationReason(code=revocation_reason,
                                           message=revocation_message)
         uuid = None
         if unique_identifier is not None:
             uuid = attr.UniqueIdentifier(unique_identifier)
 
-        payload = revoke.RevokeRequestPayload(
+        payload = payloads.RevokeRequestPayload(
             unique_identifier=uuid,
             revocation_reason=reason,
-            compromise_date=None)  # TODO(tim-kelsey): sort out date handling
+            compromise_occurrence_date=compromise_occurrence_date)
 
         batch_item = messages.RequestBatchItem(operation=operation,
                                                request_payload=payload)
@@ -769,7 +1209,7 @@ class KMIPProxy(KMIP):
         if object_type is None:
             raise ValueError('object_type cannot be None')
 
-        req_pl = register.RegisterRequestPayload(
+        req_pl = payloads.RegisterRequestPayload(
             object_type=object_type,
             template_attribute=template_attribute,
             secret=secret)
@@ -809,18 +1249,18 @@ class KMIPProxy(KMIP):
         objgrp = None
 
         if maximum_items is not None:
-            mxi = locate.LocateRequestPayload.MaximumItems(maximum_items)
+            mxi = payloads.LocateRequestPayload.MaximumItems(maximum_items)
         if storage_status_mask is not None:
             m = storage_status_mask
-            ssmask = locate.LocateRequestPayload.StorageStatusMask(m)
+            ssmask = payloads.LocateRequestPayload.StorageStatusMask(m)
         if object_group_member is not None:
             o = object_group_member
-            objgrp = locate.LocateRequestPayload.ObjectGroupMember(o)
+            objgrp = payloads.LocateRequestPayload.ObjectGroupMember(o)
 
-        payload = locate.LocateRequestPayload(maximum_items=mxi,
-                                              storage_status_mask=ssmask,
-                                              object_group_member=objgrp,
-                                              attributes=attributes)
+        payload = payloads.LocateRequestPayload(maximum_items=mxi,
+                                                storage_status_mask=ssmask,
+                                                object_group_member=objgrp,
+                                                attributes=attributes)
 
         batch_item = messages.RequestBatchItem(operation=operation,
                                                request_payload=payload)
@@ -846,6 +1286,43 @@ class KMIPProxy(KMIP):
                               uuids)
         return result
 
+    def _mac(self,
+             data,
+             unique_identifier=None,
+             cryptographic_parameters=None,
+             credential=None):
+        operation = Operation(OperationEnum.MAC)
+
+        req_pl = payloads.MACRequestPayload(
+            unique_identifier=attr.UniqueIdentifier(unique_identifier),
+            cryptographic_parameters=cryptographic_parameters,
+            data=objects.Data(data))
+        batch_item = messages.RequestBatchItem(operation=operation,
+                                               request_payload=req_pl)
+
+        message = self._build_request_message(credential, [batch_item])
+        self._send_message(message)
+        message = messages.ResponseMessage()
+        data = self._receive_message()
+        message.read(data)
+        batch_items = message.batch_items
+        batch_item = batch_items[0]
+        payload = batch_item.response_payload
+
+        if payload is None:
+            payload_unique_identifier = None
+            payload_mac_data = None
+        else:
+            payload_unique_identifier = payload.unique_identifier
+            payload_mac_data = payload.mac_data
+
+        result = MACResult(batch_item.result_status,
+                           batch_item.result_reason,
+                           batch_item.result_message,
+                           payload_unique_identifier,
+                           payload_mac_data)
+        return result
+
     # TODO (peter-hamilton) Augment to handle device credentials
     def _build_credential(self):
         if (self.username is None) and (self.password is None):
@@ -864,7 +1341,7 @@ class KMIPProxy(KMIP):
         return credential
 
     def _build_request_message(self, credential, batch_items):
-        protocol_version = ProtocolVersion.create(1, 1)
+        protocol_version = ProtocolVersion.create(1, 2)
 
         if credential is None:
             credential = self._build_credential()
