@@ -13,7 +13,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import errno
 import logging
 import logging.handlers as handlers
 import optparse
@@ -50,7 +49,12 @@ class KmipServer(object):
             ca_path=None,
             auth_suite=None,
             config_path='/etc/pykmip/server.conf',
-            log_path='/var/log/pykmip/server.log'):
+            log_path='/var/log/pykmip/server.log',
+            policy_path=None,
+            enable_tls_client_auth=None,
+            tls_cipher_suites=None,
+            logging_level=None
+    ):
         """
         Create a KmipServer.
 
@@ -91,6 +95,24 @@ class KmipServer(object):
             log_path (string): The path to the base server log file
                 (e.g., '/var/log/pykmip/server.log'). Optional, defaults to
                 '/var/log/pykmip/server.log'.
+            policy_path (string): The path to the filesystem directory
+                containing PyKMIP server operation policy JSON files.
+                Optional, defaults to None.
+            enable_tls_client_auth (boolean): A boolean indicating if the TLS
+                certificate client auth flag should be required for client
+                certificates when establishing a new client session. Optional,
+                defaults to None.
+            tls_cipher_suites (string): A comma-delimited list of cipher suite
+                names (e.g., TLS_RSA_WITH_AES_256_CBC_SHA256,TLS_RSA_WITH_AES_
+                128_CBC_SHA256), indicating which specific cipher suites should
+                be used by the server when establishing a TLS connection with
+                a client. Optional, defaults to None. If None, the default set
+                of TLS cipher suites will be used.
+            logging_level (string): A logging level enumeration defined by the
+                logging package (e.g., DEBUG, INFO). Sets the base logging
+                level for the server. All log messages logged at this level or
+                higher in criticality will be logged. All log messages lower
+                in criticality will not be logged. Optional, defaults to None.
         """
         self._logger = logging.getLogger('kmip.server')
         self._setup_logging(log_path)
@@ -103,15 +125,24 @@ class KmipServer(object):
             certificate_path,
             key_path,
             ca_path,
-            auth_suite
+            auth_suite,
+            policy_path,
+            enable_tls_client_auth,
+            tls_cipher_suites,
+            logging_level
         )
 
-        if self.config.settings.get('auth_suite') == 'TLS1.2':
-            self.auth_suite = auth.TLS12AuthenticationSuite()
-        else:
-            self.auth_suite = auth.BasicAuthenticationSuite()
+        self._logger.setLevel(self.config.settings.get('logging_level'))
 
-        self._engine = engine.KmipEngine()
+        cipher_suites = self.config.settings.get('tls_cipher_suites')
+        if self.config.settings.get('auth_suite') == 'TLS1.2':
+            self.auth_suite = auth.TLS12AuthenticationSuite(cipher_suites)
+        else:
+            self.auth_suite = auth.BasicAuthenticationSuite(cipher_suites)
+
+        self._engine = engine.KmipEngine(
+            self.config.settings.get('policy_path')
+        )
         self._session_id = 1
         self._is_serving = False
 
@@ -134,7 +165,7 @@ class KmipServer(object):
             )
         )
         self._logger.addHandler(handler)
-        self._logger.setLevel(logging.INFO)
+        self._logger.setLevel(logging.DEBUG)
 
     def _setup_configuration(
             self,
@@ -144,7 +175,12 @@ class KmipServer(object):
             certificate_path=None,
             key_path=None,
             ca_path=None,
-            auth_suite=None):
+            auth_suite=None,
+            policy_path=None,
+            enable_tls_client_auth=None,
+            tls_cipher_suites=None,
+            logging_level=None
+    ):
         if path:
             self.config.load_settings(path)
 
@@ -160,6 +196,20 @@ class KmipServer(object):
             self.config.set_setting('ca_path', ca_path)
         if auth_suite:
             self.config.set_setting('auth_suite', auth_suite)
+        if policy_path:
+            self.config.set_setting('policy_path', policy_path)
+        if enable_tls_client_auth is not None:
+            self.config.set_setting(
+                'enable_tls_client_auth',
+                enable_tls_client_auth
+            )
+        if tls_cipher_suites:
+            self.config.set_setting(
+                'tls_cipher_suites',
+                tls_cipher_suites.split(',')
+            )
+        if logging_level:
+            self.config.set_setting('logging_level', logging_level)
 
     def start(self):
         """
@@ -179,6 +229,22 @@ class KmipServer(object):
         self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         self._socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
 
+        self._logger.debug(
+            "Configured cipher suites: {0}".format(
+                len(self.config.settings.get('tls_cipher_suites'))
+            )
+        )
+        for cipher in self.config.settings.get('tls_cipher_suites'):
+            self._logger.debug(cipher)
+        auth_suite_ciphers = self.auth_suite.ciphers.split(':')
+        self._logger.debug(
+            "Authentication suite ciphers to use: {0}".format(
+                len(auth_suite_ciphers)
+            )
+        )
+        for cipher in auth_suite_ciphers:
+            self._logger.debug(cipher)
+
         self._socket = ssl.wrap_socket(
             self._socket,
             keyfile=self.config.settings.get('key_path'),
@@ -275,6 +341,13 @@ class KmipServer(object):
         def _signal_handler(signal_number, stack_frame):
             self._is_serving = False
 
+            # Python3.5+ silently ignores SIGINT and retries system calls if
+            # the signal handler does not raise an exception. Explicitly
+            # detect SIGINT and raise a KeyboardInterrupt exception to regain
+            # old functionality.
+            if signal_number == signal.SIGINT:
+                raise KeyboardInterrupt("SIGINT received")
+
         signal.signal(signal.SIGINT, _signal_handler)
         signal.signal(signal.SIGTERM, _signal_handler)
 
@@ -284,13 +357,13 @@ class KmipServer(object):
             try:
                 connection, address = self._socket.accept()
             except socket.error as e:
-                if e.errno == errno.EINTR:
-                    self._logger.warning("Interrupting connection service.")
-                else:
-                    self._logger.warning(
-                        "Error detected while establishing new connection."
-                    )
-                    self._logger.exception(e)
+                self._logger.warning(
+                    "Error detected while establishing new connection."
+                )
+                self._logger.exception(e)
+            except KeyboardInterrupt:
+                self._logger.warning("Interrupting connection service.")
+                self._is_serving = False
                 break
             except Exception as e:
                 self._logger.warning(
@@ -325,7 +398,10 @@ class KmipServer(object):
             s = session.KmipSession(
                 self._engine,
                 connection,
-                name=session_name
+                name=session_name,
+                enable_tls_client_auth=self.config.settings.get(
+                    'enable_tls_client_auth'
+                )
             )
             s.daemon = True
             s.start()
@@ -355,12 +431,12 @@ def build_argument_parser():
         "--hostname",
         action="store",
         type="str",
-        default="127.0.0.1",
+        default=None,
         dest="hostname",
         help=(
             "The host address the server will be bound to. A string "
             "representing either a hostname in Internet domain notation or "
-            "an IPv4 address. Defaults to '127.0.0.1'."
+            "an IPv4 address. Defaults to None."
         ),
     )
     parser.add_option(
@@ -368,12 +444,12 @@ def build_argument_parser():
         "--port",
         action="store",
         type="int",
-        default=5696,
+        default=None,
         dest="port",
         help=(
             "The port number the server will be bound to. An integer "
             "representing a port number. Recommended to be 5696 according to "
-            "the KMIP specification. Defaults to 5696."
+            "the KMIP specification. Defaults to None."
         ),
     )
     parser.add_option(
@@ -417,11 +493,12 @@ def build_argument_parser():
         "--auth_suite",
         action="store",
         type="str",
-        default="Basic",
+        default=None,
         dest="auth_suite",
         help=(
             "A string representing the type of authentication suite to use "
-            "when establishing TLS connections. Defaults to 'Basic'."
+            "when establishing TLS connections. Options include 'Basic' and "
+            "'TLS1.2'. Defaults to None."
         ),
     )
     parser.add_option(
@@ -447,6 +524,42 @@ def build_argument_parser():
             "A string representing a path to a log file. Defaults to None."
         ),
     )
+    parser.add_option(
+        "-o",
+        "--policy_path",
+        action="store",
+        type="str",
+        default=None,
+        dest="policy_path",
+        help=(
+            "A string representing a path to the operation policy filesystem "
+            "directory. Optional, defaults to None."
+        ),
+    )
+    parser.add_option(
+        "-i",
+        "--ignore_tls_client_auth",
+        action="store_true",
+        default=False,
+        dest="ignore_tls_client_auth",
+        help=(
+            "A boolean indicating whether or not the TLS certificate client "
+            "auth flag should be ignored when establishing client sessions. "
+            "Optional, defaults to None."
+        )
+    )
+    parser.add_option(
+        "-v",
+        "--logging_level",
+        action="store",
+        type="str",
+        default=None,
+        dest="logging_level",
+        help=(
+            "A string representing the logging level for the server (e.g., "
+            "DEBUG, INFO). Optional, defaults to None."
+        )
+    )
 
     return parser
 
@@ -473,6 +586,12 @@ def main(args=None):
         kwargs['config_path'] = opts.config_path
     if opts.log_path:
         kwargs['log_path'] = opts.log_path
+    if opts.policy_path:
+        kwargs['policy_path'] = opts.policy_path
+    if opts.ignore_tls_client_auth:
+        kwargs['enable_tls_client_auth'] = False
+    if opts.logging_level:
+        kwargs['logging_level'] = opts.logging_level
 
     # Create and start the server.
     s = KmipServer(**kwargs)
