@@ -13,6 +13,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import logging
 import mock
 
 from six.moves import configparser
@@ -38,7 +39,16 @@ class TestKmipServerConfig(testtools.TestCase):
         """
         Test that a KmipServerConfig object can be created without error.
         """
-        config.KmipServerConfig()
+        c = config.KmipServerConfig()
+
+        self.assertIn('enable_tls_client_auth', c.settings.keys())
+        self.assertEqual(
+            True,
+            c.settings.get('enable_tls_client_auth')
+        )
+
+        self.assertIn('tls_cipher_suites', c.settings.keys())
+        self.assertEqual([], c.settings.get('tls_cipher_suites'))
 
     def test_set_setting(self):
         """
@@ -53,6 +63,10 @@ class TestKmipServerConfig(testtools.TestCase):
         c._set_hostname = mock.MagicMock()
         c._set_key_path = mock.MagicMock()
         c._set_port = mock.MagicMock()
+        c._set_policy_path = mock.MagicMock()
+        c._set_enable_tls_client_auth = mock.MagicMock()
+        c._set_tls_cipher_suites = mock.MagicMock()
+        c._set_logging_level = mock.MagicMock()
 
         # Test the right error is generated when setting an unsupported
         # setting.
@@ -88,6 +102,18 @@ class TestKmipServerConfig(testtools.TestCase):
         c.set_setting('auth_suite', 'Basic')
         c._set_auth_suite.assert_called_once_with('Basic')
 
+        c.set_setting('policy_path', '/etc/pykmip/policies')
+        c._set_policy_path.assert_called_once_with('/etc/pykmip/policies')
+
+        c.set_setting('enable_tls_client_auth', False)
+        c._set_enable_tls_client_auth.assert_called_once_with(False)
+
+        c.set_setting('tls_cipher_suites', [])
+        c._set_tls_cipher_suites.assert_called_once_with([])
+
+        c.set_setting('logging_level', 'WARNING')
+        c._set_logging_level.assert_called_once_with('WARNING')
+
     def test_load_settings(self):
         """
         Test that the right calls are made and the right errors generated when
@@ -139,6 +165,10 @@ class TestKmipServerConfig(testtools.TestCase):
         c._set_hostname = mock.MagicMock()
         c._set_key_path = mock.MagicMock()
         c._set_port = mock.MagicMock()
+        c._set_policy_path = mock.MagicMock()
+        c._set_enable_tls_client_auth = mock.MagicMock()
+        c._set_tls_cipher_suites = mock.MagicMock()
+        c._set_logging_level = mock.MagicMock()
 
         # Test that the right calls are made when correctly parsing settings.
         parser = configparser.SafeConfigParser()
@@ -149,6 +179,14 @@ class TestKmipServerConfig(testtools.TestCase):
         parser.set('server', 'key_path', '/test/path/server.key')
         parser.set('server', 'ca_path', '/test/path/ca.crt')
         parser.set('server', 'auth_suite', 'Basic')
+        parser.set('server', 'policy_path', '/test/path/policies')
+        parser.set('server', 'enable_tls_client_auth', 'False')
+        parser.set(
+            'server',
+            'tls_cipher_suites',
+            "\n    TLS_RSA_WITH_AES_256_CBC_SHA256"
+        )
+        parser.set('server', 'logging_level', 'ERROR')
 
         c._parse_settings(parser)
 
@@ -160,6 +198,12 @@ class TestKmipServerConfig(testtools.TestCase):
         c._set_key_path.assert_called_once_with('/test/path/server.key')
         c._set_ca_path.assert_called_once_with('/test/path/ca.crt')
         c._set_auth_suite.assert_called_once_with('Basic')
+        c._set_policy_path.assert_called_once_with('/test/path/policies')
+        c._set_enable_tls_client_auth.assert_called_once_with(False)
+        c._set_tls_cipher_suites.assert_called_once_with(
+            "\n    TLS_RSA_WITH_AES_256_CBC_SHA256"
+        )
+        c._set_logging_level.assert_called_once_with('ERROR')
 
         # Test that a ConfigurationError is generated when the expected
         # section is missing.
@@ -478,3 +522,237 @@ class TestKmipServerConfig(testtools.TestCase):
             *args
         )
         self.assertNotEqual('invalid', c.settings.get('auth_suite'))
+
+    def test_set_policy_path(self):
+        """
+        Test that the policy_path configuration property can be set correctly.
+        """
+        c = config.KmipServerConfig()
+        c._logger = mock.MagicMock()
+
+        self.assertNotIn('policy_path', c.settings.keys())
+
+        # Test that the setting is set correctly with a valid value.
+        with mock.patch('os.path.exists') as os_mock:
+            os_mock.return_value = True
+            c._set_policy_path('/test/path/policies')
+
+        self.assertIn('policy_path', c.settings.keys())
+        self.assertEqual(
+            '/test/path/policies',
+            c.settings.get('policy_path')
+        )
+
+        c._set_policy_path(None)
+        self.assertIn('policy_path', c.settings.keys())
+        self.assertEqual(None, c.settings.get('policy_path'))
+
+        # Test that a ConfigurationError is generated when setting the wrong
+        # value.
+        c._logger.reset_mock()
+        args = (1, )
+        self.assertRaises(
+            exceptions.ConfigurationError,
+            c._set_policy_path,
+            *args
+        )
+        self.assertNotEqual(1, c.settings.get('policy_path'))
+
+    def test_set_enable_tls_client_auth(self):
+        """
+        Test that the enable_tls_client_auth configuration property can be set
+        correctly.
+        """
+        c = config.KmipServerConfig()
+        c._logger = mock.MagicMock()
+
+        # Test that the setting is set correctly with a valid value
+        c._set_enable_tls_client_auth(False)
+        self.assertEqual(
+            False,
+            c.settings.get('enable_tls_client_auth')
+        )
+
+        c._set_enable_tls_client_auth(None)
+        self.assertEqual(
+            True,
+            c.settings.get('enable_tls_client_auth')
+        )
+
+        c._set_enable_tls_client_auth(True)
+        self.assertEqual(
+            True,
+            c.settings.get('enable_tls_client_auth')
+        )
+
+        # Test that a ConfigurationError is generated when setting the wrong
+        # value.
+        args = ('invalid',)
+        self.assertRaisesRegexp(
+            exceptions.ConfigurationError,
+            "The flag enabling the TLS certificate client auth flag check "
+            "must be a boolean.",
+            c._set_enable_tls_client_auth,
+            *args
+        )
+
+    def test_set_tls_cipher_suites(self):
+        """
+        Test that the tls_cipher_suites configuration property can be set
+        correctly with a value expected from the config file.
+        """
+        c = config.KmipServerConfig()
+        c._logger = mock.MagicMock()
+
+        c._set_tls_cipher_suites(
+            """
+    TLS_RSA_WITH_AES_256_CBC_SHA256
+    TLS_RSA_WITH_AES_128_CBC_SHA256
+"""
+        )
+        self.assertEqual(2, len(c.settings.get('tls_cipher_suites')))
+        self.assertIn(
+            'TLS_RSA_WITH_AES_256_CBC_SHA256',
+            c.settings.get('tls_cipher_suites')
+        )
+        self.assertIn(
+            'TLS_RSA_WITH_AES_128_CBC_SHA256',
+            c.settings.get('tls_cipher_suites')
+        )
+
+    def test_set_tls_cipher_suites_preparsed(self):
+        """
+        Test that the tls_cipher_suites configuration property can be set
+        correctly with a preparsed list of TLS cipher suites, the value
+        expected if the cipher suites were provided via constructor.
+        """
+        c = config.KmipServerConfig()
+        c._logger = mock.MagicMock()
+
+        c._set_tls_cipher_suites(
+            [
+                'TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA',
+                'TLS_ECDH_RSA_WITH_AES_256_CBC_SHA384',
+                'DH-DSS-AES128-SHA'
+            ]
+        )
+        self.assertEqual(3, len(c.settings.get('tls_cipher_suites')))
+        self.assertIn(
+            'DH-DSS-AES128-SHA',
+            c.settings.get('tls_cipher_suites')
+        )
+        self.assertIn(
+            'TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA',
+            c.settings.get('tls_cipher_suites')
+        )
+        self.assertIn(
+            'TLS_ECDH_RSA_WITH_AES_256_CBC_SHA384',
+            c.settings.get('tls_cipher_suites')
+        )
+
+    def test_set_tls_cipher_suites_empty(self):
+        """
+        Test that the tls_cipher_suites configuration property can be set
+        correctly with an empty value.
+        """
+        c = config.KmipServerConfig()
+        c._logger = mock.MagicMock()
+
+        c._set_tls_cipher_suites(None)
+        self.assertEqual([], c.settings.get('tls_cipher_suites'))
+
+    def test_set_tls_cipher_suites_invalid_value(self):
+        """
+        Test that the right error is raised when an invalid value is used to
+        set the tls_cipher_suites configuration property.
+        """
+        c = config.KmipServerConfig()
+        c._logger = mock.MagicMock()
+
+        args = (1,)
+        self.assertRaisesRegexp(
+            exceptions.ConfigurationError,
+            "The TLS cipher suites must be a set of strings representing "
+            "cipher suite names.",
+            c._set_tls_cipher_suites,
+            *args
+        )
+
+    def test_set_tls_cipher_suites_invalid_list_value(self):
+        """
+        Test that the right error is raised when an invalid list value is used
+        to set the tls_cipher_suites configuration property.
+        """
+        c = config.KmipServerConfig()
+        c._logger = mock.MagicMock()
+
+        args = ([0],)
+        self.assertRaisesRegexp(
+            exceptions.ConfigurationError,
+            "The TLS cipher suites must be a set of strings representing "
+            "cipher suite names.",
+            c._set_tls_cipher_suites,
+            *args
+        )
+
+    def test_set_logging_level(self):
+        """
+        Test that the logging_level configuration property can be set
+        correctly with a value expected from the config file.
+        """
+        c = config.KmipServerConfig()
+        c._logger = mock.MagicMock()
+
+        c._set_logging_level("DEBUG")
+        self.assertEqual(logging.DEBUG, c.settings.get('logging_level'))
+
+    def test_set_logging_level_enum(self):
+        """
+        Test that the logging_level configuration property can be set
+        correctly with a value expected from the server constructor.
+        """
+        c = config.KmipServerConfig()
+        c._logger = mock.MagicMock()
+
+        c._set_logging_level(logging.DEBUG)
+        self.assertEqual(logging.DEBUG, c.settings.get('logging_level'))
+
+    def test_set_logging_level_default(self):
+        """
+        Test that the logging_level configuration property can be set
+        correctly without specifying a value.
+        """
+        c = config.KmipServerConfig()
+        c._logger = mock.MagicMock()
+
+        c._set_logging_level(logging.DEBUG)
+        self.assertEqual(logging.DEBUG, c.settings.get('logging_level'))
+
+        c._set_logging_level(None)
+        self.assertEqual(logging.INFO, c.settings.get('logging_level'))
+
+    def test_set_logging_level_invalid_value(self):
+        """
+        Test that the right error is raised when an invalid value is used to
+        set the tls_cipher_suites configuration property.
+        """
+        c = config.KmipServerConfig()
+        c._logger = mock.MagicMock()
+
+        args = (0,)
+        self.assertRaisesRegexp(
+            exceptions.ConfigurationError,
+            "The logging level must be a string representing a valid logging "
+            "level.",
+            c._set_logging_level,
+            *args
+        )
+
+        args = ('invalid',)
+        self.assertRaisesRegexp(
+            exceptions.ConfigurationError,
+            "The logging level must be a string representing a valid logging "
+            "level.",
+            c._set_logging_level,
+            *args
+        )
