@@ -15,8 +15,11 @@
 
 from testtools import TestCase
 
+from kmip.core.attributes import CryptographicParameters
+from kmip.core.attributes import DerivationParameters
 from kmip.core.attributes import PrivateKeyUniqueIdentifier
 
+from kmip.core import enums
 from kmip.core.enums import AuthenticationSuite
 from kmip.core.enums import ConformanceClause
 from kmip.core.enums import CredentialType
@@ -24,6 +27,8 @@ from kmip.core.enums import ResultStatus as ResultStatusEnum
 from kmip.core.enums import ResultReason as ResultReasonEnum
 from kmip.core.enums import Operation as OperationEnum
 from kmip.core.enums import QueryFunction as QueryFunctionEnum
+from kmip.core.enums import CryptographicAlgorithm as \
+                            CryptographicAlgorithmEnum
 
 from kmip.core.factories.attributes import AttributeFactory
 from kmip.core.factories.credentials import CredentialFactory
@@ -37,21 +42,14 @@ from kmip.core.messages.contents import ResultStatus
 from kmip.core.messages.contents import ResultReason
 from kmip.core.messages.contents import ResultMessage
 from kmip.core.messages.contents import ProtocolVersion
-from kmip.core.messages.payloads.create_key_pair import \
-    CreateKeyPairRequestPayload, CreateKeyPairResponsePayload
-from kmip.core.messages.payloads.discover_versions import \
-    DiscoverVersionsRequestPayload, DiscoverVersionsResponsePayload
-from kmip.core.messages.payloads import get_attribute_list
-from kmip.core.messages.payloads.query import \
-    QueryRequestPayload, QueryResponsePayload
-from kmip.core.messages.payloads.rekey_key_pair import \
-    RekeyKeyPairRequestPayload, RekeyKeyPairResponsePayload
+from kmip.core.messages import payloads
 
 from kmip.core.misc import Offset
 from kmip.core.misc import QueryFunction
 from kmip.core.misc import ServerInformation
 from kmip.core.misc import VendorIdentification
 
+from kmip.core.objects import TemplateAttribute
 from kmip.core.objects import CommonTemplateAttribute
 from kmip.core.objects import PrivateKeyTemplateAttribute
 from kmip.core.objects import PublicKeyTemplateAttribute
@@ -60,6 +58,7 @@ from kmip.services.kmip_client import KMIPProxy
 
 from kmip.services.results import CreateKeyPairResult
 from kmip.services.results import DiscoverVersionsResult
+from kmip.services.results import GetAttributesResult
 from kmip.services.results import GetAttributeListResult
 from kmip.services.results import OperationResult
 from kmip.services.results import QueryResult
@@ -97,6 +96,50 @@ class TestKMIPClient(TestCase):
     def tearDown(self):
         super(TestKMIPClient, self).tearDown()
 
+    def test_close(self):
+        """
+        Test that calling close on the client works as expected.
+        """
+        c = KMIPProxy(
+            host="IP_ADDR_1, IP_ADDR_2",
+            port=9090,
+            ca_certs=None
+        )
+        c.socket = mock.MagicMock()
+        c_socket = c.socket
+
+        c.socket.shutdown.assert_not_called()
+        c.socket.close.assert_not_called()
+
+        c.close()
+
+        self.assertEqual(None, c.socket)
+        c_socket.shutdown.assert_called_once_with(socket.SHUT_RDWR)
+        c_socket.close.assert_called_once()
+
+    def test_close_with_shutdown_error(self):
+        """
+        Test that calling close on an unconnected client does not trigger an
+        exception.
+        """
+        c = KMIPProxy(
+            host="IP_ADDR_1, IP_ADDR_2",
+            port=9090,
+            ca_certs=None
+        )
+        c.socket = mock.MagicMock()
+        c_socket = c.socket
+        c.socket.shutdown.side_effect = OSError
+
+        c.socket.shutdown.assert_not_called()
+        c.socket.close.assert_not_called()
+
+        c.close()
+
+        self.assertEqual(None, c.socket)
+        c_socket.shutdown.assert_called_once_with(socket.SHUT_RDWR)
+        c_socket.close.assert_not_called()
+
     # TODO (peter-hamilton) Modify for credential type and/or add new test
     def test_build_credential(self):
         username = 'username'
@@ -179,8 +222,12 @@ class TestKMIPClient(TestCase):
 
         payload = batch_item.request_payload
 
-        msg = base.format(CreateKeyPairRequestPayload, payload)
-        self.assertIsInstance(payload, CreateKeyPairRequestPayload, msg)
+        msg = base.format(payloads.CreateKeyPairRequestPayload, payload)
+        self.assertIsInstance(
+            payload,
+            payloads.CreateKeyPairRequestPayload,
+            msg
+        )
 
         common_observed = payload.common_template_attribute
         private_observed = payload.private_key_template_attribute
@@ -228,8 +275,12 @@ class TestKMIPClient(TestCase):
 
         payload = batch_item.request_payload
 
-        msg = base.format(RekeyKeyPairRequestPayload, payload)
-        self.assertIsInstance(payload, RekeyKeyPairRequestPayload, msg)
+        msg = base.format(payloads.RekeyKeyPairRequestPayload, payload)
+        self.assertIsInstance(
+            payload,
+            payloads.RekeyKeyPairRequestPayload,
+            msg
+        )
 
         private_key_uuid_observed = payload.private_key_uuid
         offset_observed = payload.offset
@@ -285,8 +336,8 @@ class TestKMIPClient(TestCase):
         if query_functions is None:
             query_functions = list()
 
-        msg = base.format(QueryRequestPayload, payload)
-        self.assertIsInstance(payload, QueryRequestPayload, msg)
+        msg = base.format(payloads.QueryRequestPayload, payload)
+        self.assertIsInstance(payload, payloads.QueryRequestPayload, msg)
 
         query_functions_observed = payload.query_functions
         self.assertEqual(query_functions, query_functions_observed)
@@ -321,8 +372,12 @@ class TestKMIPClient(TestCase):
         if protocol_versions is None:
             protocol_versions = list()
 
-        msg = base.format(DiscoverVersionsRequestPayload, payload)
-        self.assertIsInstance(payload, DiscoverVersionsRequestPayload, msg)
+        msg = base.format(payloads.DiscoverVersionsRequestPayload, payload)
+        self.assertIsInstance(
+            payload,
+            payloads.DiscoverVersionsRequestPayload,
+            msg
+        )
 
         observed = payload.protocol_versions
 
@@ -337,6 +392,33 @@ class TestKMIPClient(TestCase):
         protocol_versions = None
         self._test_build_discover_versions_batch_item(protocol_versions)
 
+    def test_build_get_attributes_batch_item(self):
+        uuid = '00000000-1111-2222-3333-444444444444'
+        attribute_names = [
+            'Name',
+            'Object Type'
+        ]
+        batch_item = self.client._build_get_attributes_batch_item(
+            uuid,
+            attribute_names
+        )
+
+        self.assertIsInstance(batch_item, RequestBatchItem)
+        self.assertIsInstance(batch_item.operation, Operation)
+        self.assertEqual(
+            OperationEnum.GET_ATTRIBUTES,
+            batch_item.operation.value
+        )
+        self.assertIsInstance(
+            batch_item.request_payload,
+            payloads.GetAttributesRequestPayload
+        )
+        self.assertEqual(uuid, batch_item.request_payload.unique_identifier)
+        self.assertEqual(
+            attribute_names,
+            batch_item.request_payload.attribute_names
+        )
+
     def test_build_get_attribute_list_batch_item(self):
         uid = '00000000-1111-2222-3333-444444444444'
         batch_item = self.client._build_get_attribute_list_batch_item(uid)
@@ -347,13 +429,13 @@ class TestKMIPClient(TestCase):
             OperationEnum.GET_ATTRIBUTE_LIST, batch_item.operation.value)
         self.assertIsInstance(
             batch_item.request_payload,
-            get_attribute_list.GetAttributeListRequestPayload)
-        self.assertEqual(uid, batch_item.request_payload.uid)
+            payloads.GetAttributeListRequestPayload)
+        self.assertEqual(uid, batch_item.request_payload.unique_identifier)
 
     def test_process_batch_items(self):
         batch_item = ResponseBatchItem(
             operation=Operation(OperationEnum.CREATE_KEY_PAIR),
-            response_payload=CreateKeyPairResponsePayload())
+            response_payload=payloads.CreateKeyPairResponsePayload())
         response = ResponseMessage(batch_items=[batch_item, batch_item])
         results = self.client._process_batch_items(response)
 
@@ -420,6 +502,12 @@ class TestKMIPClient(TestCase):
                                 self.client._get_batch_item_processor,
                                 0xA5A5A5A5)
 
+        expected = self.client._process_get_attributes_batch_item
+        observed = self.client._get_batch_item_processor(
+            OperationEnum.GET_ATTRIBUTES
+        )
+        self.assertEqual(expected, observed)
+
         expected = self.client._process_get_attribute_list_batch_item
         observed = self.client._get_batch_item_processor(
             OperationEnum.GET_ATTRIBUTE_LIST)
@@ -432,7 +520,7 @@ class TestKMIPClient(TestCase):
     def test_process_create_key_pair_batch_item(self):
         batch_item = ResponseBatchItem(
             operation=Operation(OperationEnum.CREATE_KEY_PAIR),
-            response_payload=CreateKeyPairResponsePayload())
+            response_payload=payloads.CreateKeyPairResponsePayload())
         result = self.client._process_create_key_pair_batch_item(batch_item)
 
         msg = "expected {0}, received {1}".format(CreateKeyPairResult, result)
@@ -441,7 +529,7 @@ class TestKMIPClient(TestCase):
     def test_process_rekey_key_pair_batch_item(self):
         batch_item = ResponseBatchItem(
             operation=Operation(OperationEnum.REKEY_KEY_PAIR),
-            response_payload=RekeyKeyPairResponsePayload())
+            response_payload=payloads.RekeyKeyPairResponsePayload())
         result = self.client._process_rekey_key_pair_batch_item(batch_item)
 
         msg = "expected {0}, received {1}".format(RekeyKeyPairResult, result)
@@ -456,7 +544,7 @@ class TestKMIPClient(TestCase):
             application_namespaces,
             extension_information):
 
-        payload = QueryResponsePayload(
+        payload = payloads.QueryResponsePayload(
             operations,
             object_types,
             vendor_identification,
@@ -508,7 +596,7 @@ class TestKMIPClient(TestCase):
     def _test_process_discover_versions_batch_item(self, protocol_versions):
         batch_item = ResponseBatchItem(
             operation=Operation(OperationEnum.DISCOVER_VERSIONS),
-            response_payload=DiscoverVersionsResponsePayload(
+            response_payload=payloads.DiscoverVersionsResponsePayload(
                 protocol_versions))
         result = self.client._process_discover_versions_batch_item(batch_item)
 
@@ -531,11 +619,28 @@ class TestKMIPClient(TestCase):
         protocol_versions = None
         self._test_process_discover_versions_batch_item(protocol_versions)
 
+    def test_process_get_attributes_batch_item(self):
+        uuid = '00000000-1111-2222-3333-444444444444'
+        attributes = []
+        payload = payloads.GetAttributesResponsePayload(
+            unique_identifier=uuid,
+            attributes=attributes
+        )
+        batch_item = ResponseBatchItem(
+            operation=Operation(OperationEnum.GET_ATTRIBUTES),
+            response_payload=payload
+        )
+        result = self.client._process_get_attributes_batch_item(batch_item)
+
+        self.assertIsInstance(result, GetAttributesResult)
+        self.assertEqual(uuid, result.uuid)
+        self.assertEqual(attributes, result.attributes)
+
     def test_process_get_attribute_list_batch_item(self):
         uid = '00000000-1111-2222-3333-444444444444'
         names = ['Cryptographic Algorithm', 'Cryptographic Length']
-        payload = get_attribute_list.GetAttributeListResponsePayload(
-            uid=uid, attribute_names=names)
+        payload = payloads.GetAttributeListResponsePayload(
+            unique_identifier=uid, attribute_names=names)
         batch_item = ResponseBatchItem(
             operation=Operation(OperationEnum.GET_ATTRIBUTE_LIST),
             response_payload=payload)
@@ -618,6 +723,373 @@ class TestKMIPClient(TestCase):
         self.client._create_socket(sock)
         self.assertEqual(ssl.SSLSocket, type(self.client.socket))
 
+    @mock.patch(
+        'kmip.services.kmip_client.KMIPProxy._build_request_message'
+    )
+    @mock.patch(
+        'kmip.services.kmip_client.KMIPProxy._send_and_receive_message'
+    )
+    def test_derive_key(self, send_mock, build_mock):
+        """
+        Test that the client can derive a key.
+        """
+        payload = payloads.DeriveKeyResponsePayload(
+            unique_identifier='1',
+        )
+        batch_item = ResponseBatchItem(
+            operation=Operation(OperationEnum.DERIVE_KEY),
+            result_status=ResultStatus(ResultStatusEnum.SUCCESS),
+            response_payload=payload
+        )
+        response = ResponseMessage(batch_items=[batch_item])
+
+        build_mock.return_value = None
+        send_mock.return_value = response
+
+        result = self.client.derive_key(
+            object_type=enums.ObjectType.SYMMETRIC_KEY,
+            unique_identifiers=['2', '3'],
+            derivation_method=enums.DerivationMethod.ENCRYPT,
+            derivation_parameters=DerivationParameters(
+                cryptographic_parameters=CryptographicParameters(
+                    block_cipher_mode=enums.BlockCipherMode.CBC,
+                    padding_method=enums.PaddingMethod.PKCS1v15,
+                    cryptographic_algorithm=enums.CryptographicAlgorithm.AES
+                ),
+                initialization_vector=b'\x01\x02\x03\x04',
+                derivation_data=b'\xF1\xF2\xF3\xF4\xF5\xF6\xF7\xF8'
+            ),
+            template_attribute=TemplateAttribute(
+                attributes=[
+                    self.attr_factory.create_attribute(
+                        'Cryptographic Length',
+                        128
+                    ),
+                    self.attr_factory.create_attribute(
+                        'Cryptographic Algorithm',
+                        enums.CryptographicAlgorithm.AES
+                    )
+                ]
+            ),
+        )
+
+        self.assertEqual('1', result.get('unique_identifier'))
+        self.assertEqual(
+            ResultStatusEnum.SUCCESS,
+            result.get('result_status')
+        )
+        self.assertEqual(None, result.get('result_reason'))
+        self.assertEqual(None, result.get('result_message'))
+
+    @mock.patch(
+        'kmip.services.kmip_client.KMIPProxy._build_request_message'
+    )
+    @mock.patch(
+        'kmip.services.kmip_client.KMIPProxy._send_and_receive_message'
+    )
+    def test_encrypt(self, send_mock, build_mock):
+        """
+        Test that the client can encrypt data.
+        """
+        payload = payloads.EncryptResponsePayload(
+            unique_identifier='1',
+            data=(
+                b'\x6B\x77\xB4\xD6\x30\x06\xDE\xE6'
+                b'\x05\xB1\x56\xE2\x74\x03\x97\x93'
+                b'\x58\xDE\xB9\xE7\x15\x46\x16\xD9'
+                b'\x74\x9D\xEC\xBE\xC0\x5D\x26\x4B'
+            )
+        )
+        batch_item = ResponseBatchItem(
+            operation=Operation(OperationEnum.ENCRYPT),
+            result_status=ResultStatus(ResultStatusEnum.SUCCESS),
+            response_payload=payload
+        )
+        response = ResponseMessage(batch_items=[batch_item])
+
+        build_mock.return_value = None
+        send_mock.return_value = response
+
+        result = self.client.encrypt(
+            (
+                b'\x37\x36\x35\x34\x33\x32\x31\x20'
+                b'\x4E\x6F\x77\x20\x69\x73\x20\x74'
+                b'\x68\x65\x20\x74\x69\x6D\x65\x20'
+                b'\x66\x6F\x72\x20\x00'
+            ),
+            unique_identifier='1',
+            cryptographic_parameters=CryptographicParameters(
+                block_cipher_mode=enums.BlockCipherMode.CBC,
+                padding_method=enums.PaddingMethod.PKCS5,
+                cryptographic_algorithm=enums.CryptographicAlgorithm.BLOWFISH
+            ),
+            iv_counter_nonce=b'\xFE\xDC\xBA\x98\x76\x54\x32\x10'
+        )
+
+        self.assertEqual('1', result.get('unique_identifier'))
+        self.assertEqual(
+            (
+                b'\x6B\x77\xB4\xD6\x30\x06\xDE\xE6'
+                b'\x05\xB1\x56\xE2\x74\x03\x97\x93'
+                b'\x58\xDE\xB9\xE7\x15\x46\x16\xD9'
+                b'\x74\x9D\xEC\xBE\xC0\x5D\x26\x4B'
+            ),
+            result.get('data')
+        )
+        self.assertEqual(None, result.get('iv_counter_nonce'))
+        self.assertEqual(
+            ResultStatusEnum.SUCCESS,
+            result.get('result_status')
+        )
+        self.assertEqual(None, result.get('result_reason'))
+        self.assertEqual(None, result.get('result_message'))
+
+    @mock.patch(
+        'kmip.services.kmip_client.KMIPProxy._build_request_message'
+    )
+    @mock.patch(
+        'kmip.services.kmip_client.KMIPProxy._send_and_receive_message'
+    )
+    def test_decrypt(self, send_mock, build_mock):
+        """
+        Test that the client can decrypt data.
+        """
+        payload = payloads.DecryptResponsePayload(
+            unique_identifier='1',
+            data=(
+                b'\x37\x36\x35\x34\x33\x32\x31\x20'
+                b'\x4E\x6F\x77\x20\x69\x73\x20\x74'
+                b'\x68\x65\x20\x74\x69\x6D\x65\x20'
+                b'\x66\x6F\x72\x20\x00'
+            )
+        )
+        batch_item = ResponseBatchItem(
+            operation=Operation(OperationEnum.DECRYPT),
+            result_status=ResultStatus(ResultStatusEnum.SUCCESS),
+            response_payload=payload
+        )
+        response = ResponseMessage(batch_items=[batch_item])
+
+        build_mock.return_value = None
+        send_mock.return_value = response
+
+        result = self.client.decrypt(
+            (
+                b'\x6B\x77\xB4\xD6\x30\x06\xDE\xE6'
+                b'\x05\xB1\x56\xE2\x74\x03\x97\x93'
+                b'\x58\xDE\xB9\xE7\x15\x46\x16\xD9'
+                b'\x74\x9D\xEC\xBE\xC0\x5D\x26\x4B'
+            ),
+            unique_identifier='1',
+            cryptographic_parameters=CryptographicParameters(
+                block_cipher_mode=enums.BlockCipherMode.CBC,
+                padding_method=enums.PaddingMethod.PKCS5,
+                cryptographic_algorithm=enums.CryptographicAlgorithm.BLOWFISH
+            ),
+            iv_counter_nonce=b'\xFE\xDC\xBA\x98\x76\x54\x32\x10'
+        )
+
+        self.assertEqual('1', result.get('unique_identifier'))
+        self.assertEqual(
+            (
+                b'\x37\x36\x35\x34\x33\x32\x31\x20'
+                b'\x4E\x6F\x77\x20\x69\x73\x20\x74'
+                b'\x68\x65\x20\x74\x69\x6D\x65\x20'
+                b'\x66\x6F\x72\x20\x00'
+            ),
+            result.get('data')
+        )
+        self.assertEqual(
+            ResultStatusEnum.SUCCESS,
+            result.get('result_status')
+        )
+        self.assertEqual(None, result.get('result_reason'))
+        self.assertEqual(None, result.get('result_message'))
+
+    @mock.patch(
+        'kmip.services.kmip_client.KMIPProxy._build_request_message'
+    )
+    @mock.patch(
+        'kmip.services.kmip_client.KMIPProxy._send_and_receive_message'
+    )
+    def test_signature_verify(self, send_mock, build_mock):
+        """
+        Test that the client can verify a signature.
+        """
+        payload = payloads.SignatureVerifyResponsePayload(
+            unique_identifier='1',
+            validity_indicator=enums.ValidityIndicator.INVALID
+        )
+        batch_item = ResponseBatchItem(
+            operation=Operation(OperationEnum.SIGNATURE_VERIFY),
+            result_status=ResultStatus(ResultStatusEnum.SUCCESS),
+            response_payload=payload
+        )
+        response = ResponseMessage(batch_items=[batch_item])
+
+        build_mock.return_value = None
+        send_mock.return_value = response
+
+        result = self.client.signature_verify(
+            (
+                b'\x6B\x77\xB4\xD6\x30\x06\xDE\xE6'
+                b'\x05\xB1\x56\xE2\x74\x03\x97\x93'
+                b'\x58\xDE\xB9\xE7\x15\x46\x16\xD9'
+                b'\x74\x9D\xEC\xBE\xC0\x5D\x26\x4B'
+            ),
+            (
+                b'\x11\x11\x11\x11\x11\x11\x11\x11'
+            ),
+            unique_identifier='1',
+            cryptographic_parameters=CryptographicParameters(
+                padding_method=enums.PaddingMethod.PKCS1v15,
+                cryptographic_algorithm=enums.CryptographicAlgorithm.RSA,
+                hashing_algorithm=enums.HashingAlgorithm.SHA_224
+            )
+        )
+
+        self.assertEqual('1', result.get('unique_identifier'))
+        self.assertEqual(
+            enums.ValidityIndicator.INVALID,
+            result.get('validity_indicator')
+        )
+        self.assertEqual(
+            ResultStatusEnum.SUCCESS,
+            result.get('result_status')
+        )
+        self.assertEqual(None, result.get('result_reason'))
+        self.assertEqual(None, result.get('result_message'))
+
+    @mock.patch(
+        'kmip.services.kmip_client.KMIPProxy._build_request_message'
+    )
+    @mock.patch(
+        'kmip.services.kmip_client.KMIPProxy._send_and_receive_message'
+    )
+    def test_sign(self, send_mock, build_mock):
+        """
+        Test that the client can sign data
+        """
+        payload = payloads.SignResponsePayload(
+            unique_identifier='1',
+            signature_data=b'aaaaaaaaaaaaaaaa'
+        )
+        batch_item = ResponseBatchItem(
+            operation=Operation(OperationEnum.SIGN),
+            result_status=ResultStatus(ResultStatusEnum.SUCCESS),
+            response_payload=payload
+        )
+        response = ResponseMessage(batch_items=[batch_item])
+
+        build_mock.return_value = None
+        send_mock.return_value = response
+
+        result = self.client.sign(
+            b'\x11\x11\x11\x11\x11\x11\x11\x11',
+            unique_identifier='1',
+            cryptographic_parameters=CryptographicParameters(
+                padding_method=enums.PaddingMethod.PKCS1v15,
+                cryptographic_algorithm=enums.CryptographicAlgorithm.RSA,
+                hashing_algorithm=enums.HashingAlgorithm.SHA_224
+            )
+        )
+
+        self.assertEqual('1', result.get('unique_identifier'))
+        self.assertEqual(
+            b'aaaaaaaaaaaaaaaa',
+            result.get('signature')
+        )
+        self.assertEqual(
+            ResultStatusEnum.SUCCESS,
+            result.get('result_status')
+        )
+        self.assertEqual(None, result.get('result_reason'))
+        self.assertEqual(None, result.get('result_message'))
+
+    @mock.patch('kmip.services.kmip_client.KMIPProxy._send_message',
+                mock.MagicMock())
+    @mock.patch('kmip.services.kmip_client.KMIPProxy._receive_message',
+                mock.MagicMock())
+    def test_mac(self):
+
+        from kmip.core.utils import BytearrayStream
+
+        request_expected = (
+            b'\x42\x00\x78\x01\x00\x00\x00\xa0\x42\x00\x77\x01\x00\x00\x00\x38'
+            b'\x42\x00\x69\x01\x00\x00\x00\x20\x42\x00\x6a\x02\x00\x00\x00\x04'
+            b'\x00\x00\x00\x01\x00\x00\x00\x00\x42\x00\x6b\x02\x00\x00\x00\x04'
+            b'\x00\x00\x00\x02\x00\x00\x00\x00\x42\x00\x0d\x02\x00\x00\x00\x04'
+            b'\x00\x00\x00\x01\x00\x00\x00\x00\x42\x00\x0f\x01\x00\x00\x00\x58'
+            b'\x42\x00\x5c\x05\x00\x00\x00\x04\x00\x00\x00\x23\x00\x00\x00\x00'
+            b'\x42\x00\x79\x01\x00\x00\x00\x40\x42\x00\x94\x07\x00\x00\x00\x01'
+            b'\x31\x00\x00\x00\x00\x00\x00\x00\x42\x00\x2b\x01\x00\x00\x00\x10'
+            b'\x42\x00\x28\x05\x00\x00\x00\x04\x00\x00\x00\x0b\x00\x00\x00\x00'
+            b'\x42\x00\xc2\x08\x00\x00\x00\x10\x00\x01\x02\x03\x04\x05\x06\x07'
+            b'\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f')
+        response = (
+            b'\x42\x00\x7b\x01\x00\x00\x00\xd8\x42\x00\x7a\x01\x00\x00\x00\x48'
+            b'\x42\x00\x69\x01\x00\x00\x00\x20\x42\x00\x6a\x02\x00\x00\x00\x04'
+            b'\x00\x00\x00\x01\x00\x00\x00\x00\x42\x00\x6b\x02\x00\x00\x00\x04'
+            b'\x00\x00\x00\x02\x00\x00\x00\x00\x42\x00\x92\x09\x00\x00\x00\x08'
+            b'\x00\x00\x00\x00\x58\x8a\x3f\x23\x42\x00\x0d\x02\x00\x00\x00\x04'
+            b'\x00\x00\x00\x01\x00\x00\x00\x00\x42\x00\x0f\x01\x00\x00\x00\x80'
+            b'\x42\x00\x5c\x05\x00\x00\x00\x04\x00\x00\x00\x23\x00\x00\x00\x00'
+            b'\x42\x00\x7f\x05\x00\x00\x00\x04\x00\x00\x00\x00\x00\x00\x00\x00'
+            b'\x42\x00\x7c\x01\x00\x00\x00\x58\x42\x00\x94\x07\x00\x00\x00\x01'
+            b'\x31\x00\x00\x00\x00\x00\x00\x00\x42\x00\xc6\x08\x00\x00\x00\x40'
+            b'\x99\x8b\x55\x59\x90\x9b\x85\x87\x5b\x90\x63\x13\x12\xbb\x32\x9f'
+            b'\x6a\xc4\xed\x97\x6e\xac\x99\xe5\x21\x53\xc4\x19\x28\xf2\x2a\x5b'
+            b'\xef\x79\xa4\xbe\x05\x3b\x31\x49\x19\xe0\x75\x23\xb9\xbe\xc8\x23'
+            b'\x35\x60\x7e\x49\xba\xa9\x7e\xe0\x9e\x6b\x3d\x55\xf4\x51\xff\x7c'
+        )
+        response_no_payload = (
+            b'\x42\x00\x7b\x01\x00\x00\x00\x78\x42\x00\x7a\x01\x00\x00\x00\x48'
+            b'\x42\x00\x69\x01\x00\x00\x00\x20\x42\x00\x6a\x02\x00\x00\x00\x04'
+            b'\x00\x00\x00\x01\x00\x00\x00\x00\x42\x00\x6b\x02\x00\x00\x00\x04'
+            b'\x00\x00\x00\x02\x00\x00\x00\x00\x42\x00\x92\x09\x00\x00\x00\x08'
+            b'\x00\x00\x00\x00\x58\x8a\x3f\x23\x42\x00\x0d\x02\x00\x00\x00\x04'
+            b'\x00\x00\x00\x01\x00\x00\x00\x00\x42\x00\x0f\x01\x00\x00\x00\x80'
+            b'\x42\x00\x5c\x05\x00\x00\x00\x04\x00\x00\x00\x23\x00\x00\x00\x00'
+            b'\x42\x00\x7f\x05\x00\x00\x00\x04\x00\x00\x00\x00\x00\x00\x00\x00'
+        )
+
+        data = (b'\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0A\x0B'
+                b'\x0C\x0D\x0E\x0F')
+
+        mdata = (b'\x99\x8b\x55\x59\x90\x9b\x85\x87\x5b\x90\x63\x13'
+                 b'\x12\xbb\x32\x9f'
+                 b'\x6a\xc4\xed\x97\x6e\xac\x99\xe5\x21\x53\xc4\x19'
+                 b'\x28\xf2\x2a\x5b'
+                 b'\xef\x79\xa4\xbe\x05\x3b\x31\x49\x19\xe0\x75\x23'
+                 b'\xb9\xbe\xc8\x23'
+                 b'\x35\x60\x7e\x49\xba\xa9\x7e\xe0\x9e\x6b\x3d\x55'
+                 b'\xf4\x51\xff\x7c')
+
+        def verify_request(message):
+            stream = BytearrayStream()
+            message.write(stream)
+            self.assertEqual(stream.buffer, request_expected)
+
+        uuid = '1'
+
+        cryptographic_parameters = CryptographicParameters(
+            cryptographic_algorithm=CryptographicAlgorithmEnum.HMAC_SHA512
+        )
+
+        self.client._send_message.side_effect = verify_request
+        self.client._receive_message.return_value = BytearrayStream(response)
+
+        result = self.client.mac(data, uuid, cryptographic_parameters)
+        self.assertEqual(result.uuid.value, uuid)
+        self.assertEqual(result.mac_data.value, mdata)
+
+        self.client._receive_message.return_value = \
+            BytearrayStream(response_no_payload)
+
+        result = self.client.mac(data, uuid, cryptographic_parameters)
+        self.assertEqual(result.uuid, None)
+        self.assertEqual(result.mac_data, None)
+
 
 class TestClientProfileInformation(TestCase):
     """
